@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2011 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2011,2015 Olly Betts
  * Copyright 2008 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -32,6 +32,7 @@
 # include "msvc_posix_wrapper.h"
 #endif
 
+#include "errno_to_string.h"
 #include "omassert.h"
 #include "str.h"
 #include "stringutils.h" // For STRINGIZE().
@@ -71,6 +72,7 @@ PWRITE_PROTOTYPE
 #include "brass_cursor.h"
 
 #include "debuglog.h"
+#include "errno_to_string.h"
 #include "io_utils.h"
 #include "omassert.h"
 #include "pack.h"
@@ -207,7 +209,7 @@ BrassTable::read_block(uint4 n, byte * p) const
 	    if (errno == EBADF && handle == -2)
 		BrassTable::throw_database_closed();
 	    string message = "Error reading block " + str(n) + ": ";
-	    message += strerror(errno);
+	    errno_to_string(errno, message);
 	    throw Xapian::DatabaseError(message);
 	} else if (bytes_read == 0) {
 	    string message = "Error reading block " + str(n) + ": got end of file";
@@ -226,7 +228,7 @@ BrassTable::read_block(uint4 n, byte * p) const
 	if (errno == EBADF && handle == -2)
 	    BrassTable::throw_database_closed();
 	string message = "Error seeking to block: ";
-	message += strerror(errno);
+	errno_to_string(errno, message);
 	throw Xapian::DatabaseError(message);
     }
 
@@ -285,7 +287,7 @@ BrassTable::write_block(uint4 n, const byte * p) const
 	} else if (bytes_written == -1) {
 	    if (errno == EINTR) continue;
 	    string message = "Error writing block: ";
-	    message += strerror(errno);
+	    errno_to_string(errno, message);
 	    throw Xapian::DatabaseError(message);
 	} else if (bytes_written == 0) {
 	    string message = "Error writing block: wrote no data";
@@ -302,7 +304,7 @@ BrassTable::write_block(uint4 n, const byte * p) const
 #else
     if (lseek(handle, (off_t)block_size * n, SEEK_SET) == -1) {
 	string message = "Error seeking to block: ";
-	message += strerror(errno);
+	errno_to_string(errno, message);
 	throw Xapian::DatabaseError(message);
     }
 
@@ -673,9 +675,11 @@ BrassTable::mid_point(byte * p)
 	}
     }
 
-    /* falling out of mid_point */
+    /* This shouldn't happen, as the sum of the item sizes should be the same
+     * as the value calculated in size, so assert but return a sane value just
+     * in case. */
     Assert(false);
-    RETURN(0); /* Stop compiler complaining about end of method. */
+    RETURN(dir_end);
 }
 
 /** add_item_to_block(p, kt_, c) adds item kt_ to the block at p.
@@ -1517,7 +1521,7 @@ BrassTable::do_open_to_write(bool revision_supplied,
 	string message(create_db ? "Couldn't create " : "Couldn't open ");
 	message += name;
 	message += "DB read/write: ";
-	message += strerror(errno);
+	errno_to_string(errno, message);
 	throw Xapian::DatabaseOpeningError(message);
     }
 
@@ -1882,7 +1886,7 @@ BrassTable::commit(brass_revision_number_t revision, int changes_fd,
 		string msg("Couldn't update base file ");
 		msg += basefile;
 		msg += ": ";
-		msg += strerror(saved_errno);
+		errno_to_string(saved_errno, msg);
 		throw Xapian::DatabaseError(msg);
 	    }
 	}
@@ -2006,7 +2010,7 @@ BrassTable::do_open_to_read(bool revision_supplied, brass_revision_number_t revi
 	string message("Couldn't open ");
 	message += name;
 	message += "DB to read: ";
-	message += strerror(errno);
+	errno_to_string(errno, message);
 	throw Xapian::DatabaseOpeningError(message);
     }
 
