@@ -2,7 +2,7 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2011,2013 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2011,2013,2015 Olly Betts
  * Copyright 2008 Lemur Consulting Ltd
  * Copyright 2010 Richard Boulton
  *
@@ -33,6 +33,7 @@
 # include "msvc_posix_wrapper.h"
 #endif
 
+#include "errno_to_string.h"
 #include "omassert.h"
 #include "stringutils.h" // For STRINGIZE().
 
@@ -208,7 +209,7 @@ ChertTable::read_block(uint4 n, byte * p) const
 	    if (errno == EBADF && handle == -2)
 		ChertTable::throw_database_closed();
 	    string message = "Error reading block " + str(n) + ": ";
-	    message += strerror(errno);
+	    errno_to_string(errno, message);
 	    throw Xapian::DatabaseError(message);
 	} else if (bytes_read == 0) {
 	    string message = "Error reading block " + str(n) + ": got end of file";
@@ -227,7 +228,7 @@ ChertTable::read_block(uint4 n, byte * p) const
 	if (errno == EBADF && handle == -2)
 	    ChertTable::throw_database_closed();
 	string message = "Error seeking to block: ";
-	message += strerror(errno);
+	errno_to_string(errno, message);
 	throw Xapian::DatabaseError(message);
     }
 
@@ -286,7 +287,7 @@ ChertTable::write_block(uint4 n, const byte * p) const
 	} else if (bytes_written == -1) {
 	    if (errno == EINTR) continue;
 	    string message = "Error writing block: ";
-	    message += strerror(errno);
+	    errno_to_string(errno, message);
 	    throw Xapian::DatabaseError(message);
 	} else if (bytes_written == 0) {
 	    string message = "Error writing block: wrote no data";
@@ -303,7 +304,7 @@ ChertTable::write_block(uint4 n, const byte * p) const
 #else
     if (lseek(handle, (off_t)block_size * n, SEEK_SET) == -1) {
 	string message = "Error seeking to block: ";
-	message += strerror(errno);
+	errno_to_string(errno, message);
 	throw Xapian::DatabaseError(message);
     }
 
@@ -463,14 +464,22 @@ ChertTable::alter()
    right ends of the search area. In sequential addition, c will often
    be the answer, so we test the keys round c and move i and j towards
    c if possible.
+
+   The returned value is < DIR_END(p).  If leaf is false, the returned
+   value is >= DIR_START; if leaf is true, it can also be == DIR_START - D2.
 */
 
 int
 ChertTable::find_in_block(const byte * p, Key key, bool leaf, int c)
 {
     LOGCALL_STATIC(DB, int, "ChertTable::find_in_block", (const void*)p | (const void *)key.get_address() | leaf | c);
+    // c should be odd (either -1, or an even offset from DIR_START).
+    Assert((c & 1) == 1);
     int i = DIR_START;
     if (leaf) i -= D2;
+    if (c != -1) {
+	AssertRel(i,<=,c);
+    }
     int j = DIR_END(p);
 
     if (c != -1) {
@@ -485,6 +494,12 @@ ChertTable::find_in_block(const byte * p, Key key, bool leaf, int c)
 	int k = i + ((j - i)/(D2 * 2))*D2; /* mid way */
 	if (key < Item(p, k).key()) j = k; else i = k;
     }
+    if (leaf) {
+	AssertRel(DIR_START - D2,<=,i);
+    } else {
+	AssertRel(DIR_START,<=,i);
+    }
+    AssertRel(i,<,DIR_END(p));
     RETURN(i);
 }
 
@@ -493,6 +508,12 @@ ChertTable::find_in_block(const byte * p, Key key, bool leaf, int c)
    Result is true if found, false otherwise.  When false, the B_tree
    cursor is positioned at the last key in the B-tree <= the search
    key.  Goes to first (null) item in B-tree when key length == 0.
+
+   Note: The cursor can be left with C_[0].c == DIR_START - D2 if the
+   requested key doesn't exist and is less than the smallest key in a
+   leaf block, but after the dividing key.  The caller needs to fix up
+   C_[0].c in this case, either explicitly or by performing an
+   operation which gives C_[0].c a valid value.
 */
 
 bool
@@ -520,7 +541,9 @@ ChertTable::find(Cursor * C_) const
     report_block_full(0, C_[0].n, p);
 #endif /* BTREE_DEBUG_FULL */
     C_[0].c = c;
-    if (c < DIR_START) RETURN(false);
+    if (c < DIR_START) {
+	RETURN(false);
+    }
     RETURN(Item(p, c).key() == key);
 }
 
@@ -651,7 +674,10 @@ ChertTable::enter_key(int j, Key prevkey, Key newkey)
 	SET_TOTAL_FREE(p, new_total_free);
     }
 
-    C[j].c = find_in_block(C[j].p, item.key(), false, 0) + D2;
+    // The split block gets inserted into the parent after the pointer to the
+    // current child.
+    AssertEq(C[j].c, find_in_block(C[j].p, item.key(), false, C[j].c));
+    C[j].c += D2;
     C[j].rewrite = true; /* a subtle point: this *is* required. */
     add_item(item, j);
 }
@@ -676,9 +702,11 @@ ChertTable::mid_point(byte * p)
 	}
     }
 
-    /* falling out of mid_point */
+    /* This shouldn't happen, as the sum of the item sizes should be the same
+     * as the value calculated in size, so assert but return a sane value just
+     * in case. */
     Assert(false);
-    RETURN(0); /* Stop compiler complaining about end of method. */
+    RETURN(dir_end);
 }
 
 /** add_item_to_block(p, kt_, c) adds item kt_ to the block at p.
@@ -705,7 +733,8 @@ ChertTable::add_item_to_block(byte * p, Item_wr kt_, int c)
 
     AssertRel(MAX_FREE(p),>=,needed);
 
-    Assert(dir_end >= c);
+    AssertRel(DIR_START,<=,c);
+    AssertRel(c,<=,dir_end);
 
     memmove(p + c + D2, p + c, dir_end - c);
     dir_end += D2;
@@ -748,6 +777,7 @@ ChertTable::add_item(Item_wr kt_, int j)
 	    m = mid_point(p);
 	} else {
 	    // During sequential addition, split at the insert point
+	    AssertRel(c,>=,DIR_START);
 	    m = c;
 	}
 
@@ -830,6 +860,8 @@ ChertTable::delete_item(int j, bool repeatedly)
     Assert(writable);
     byte * p = C[j].p;
     int c = C[j].c;
+    AssertRel(DIR_START,<=,c);
+    AssertRel(c,<,DIR_END(p));
     int kt_len = Item(p, c).size(); /* size of the item to be deleted */
     int dir_end = DIR_END(p) - D2;   /* directory length will go down by 2 bytes */
 
@@ -917,6 +949,8 @@ ChertTable::add_kt(bool found)
 
 	byte * p = C[0].p;
 	int c = C[0].c;
+	AssertRel(DIR_START,<=,c);
+	AssertRel(c,<,DIR_END(p));
 	Item item(p, c);
 	int kt_size = kt.size();
 	int needed = kt_size - item.size();
@@ -1520,7 +1554,7 @@ ChertTable::do_open_to_write(bool revision_supplied,
 	string message(create_db ? "Couldn't create " : "Couldn't open ");
 	message += name;
 	message += "DB read/write: ";
-	message += strerror(errno);
+	errno_to_string(errno, message);
 	throw Xapian::DatabaseOpeningError(message);
     }
 
@@ -1903,7 +1937,7 @@ ChertTable::commit(chert_revision_number_t revision, int changes_fd,
 		string msg("Couldn't update base file ");
 		msg += basefile;
 		msg += ": ";
-		msg += strerror(saved_errno);
+		errno_to_string(saved_errno, msg);
 		throw Xapian::DatabaseError(msg);
 	    }
 	}
@@ -2028,7 +2062,7 @@ ChertTable::do_open_to_read(bool revision_supplied, chert_revision_number_t revi
 	string message("Couldn't open ");
 	message += name;
 	message += "DB to read: ";
-	message += strerror(errno);
+	errno_to_string(errno, message);
 	throw Xapian::DatabaseOpeningError(message);
     }
 
@@ -2106,6 +2140,8 @@ ChertTable::prev_for_sequential(Cursor * C_, int /*dummy*/) const
 {
     LOGCALL(DB, bool, "ChertTable::prev_for_sequential", Literal("C_") | Literal("/*dummy*/"));
     int c = C_[0].c;
+    AssertRel(DIR_START,<=,c);
+    AssertRel(c,<,DIR_END(C_[0].p));
     if (c == DIR_START) {
 	byte * p = C_[0].p;
 	Assert(p);
@@ -2147,6 +2183,7 @@ ChertTable::prev_for_sequential(Cursor * C_, int /*dummy*/) const
 	}
 	c = DIR_END(p);
 	C_[0].n = n;
+	AssertRel(DIR_START,<,c);
     }
     c -= D2;
     C_[0].c = c;
@@ -2160,6 +2197,7 @@ ChertTable::next_for_sequential(Cursor * C_, int /*dummy*/) const
     byte * p = C_[0].p;
     Assert(p);
     int c = C_[0].c;
+    AssertRel(c,<,DIR_END(p));
     c += D2;
     Assert((unsigned)c < block_size);
     if (c == DIR_END(p)) {
@@ -2212,13 +2250,14 @@ ChertTable::prev_default(Cursor * C_, int j) const
     LOGCALL(DB, bool, "ChertTable::prev_default", Literal("C_") | j);
     byte * p = C_[j].p;
     int c = C_[j].c;
-    Assert(c >= DIR_START);
-    Assert((unsigned)c < block_size);
-    Assert(c <= DIR_END(p));
+    AssertRel(DIR_START,<=,c);
+    AssertRel(c,<,DIR_END(p));
+    AssertRel((unsigned)DIR_END(p),<=,block_size);
     if (c == DIR_START) {
 	if (j == level) RETURN(false);
 	if (!prev_default(C_, j + 1)) RETURN(false);
 	c = DIR_END(p);
+	AssertRel(DIR_START,<,c);
     }
     c -= D2;
     C_[j].c = c;
@@ -2234,9 +2273,14 @@ ChertTable::next_default(Cursor * C_, int j) const
     LOGCALL(DB, bool, "ChertTable::next_default", Literal("C_") | j);
     byte * p = C_[j].p;
     int c = C_[j].c;
-    Assert(c >= DIR_START);
+    AssertRel(c,<,DIR_END(p));
+    AssertRel((unsigned)DIR_END(p),<=,block_size);
     c += D2;
-    Assert((unsigned)c < block_size);
+    if (j > 0) {
+	AssertRel(DIR_START,<,c);
+    } else {
+	AssertRel(DIR_START,<=,c);
+    }
     // Sometimes c can be DIR_END(p) + 2 here it appears...
     if (c >= DIR_END(p)) {
 	if (j == level) RETURN(false);
