@@ -80,13 +80,19 @@ TcpClient::open_socket(const std::string & hostname, int port,
 #ifdef __WIN32__
     ULONG enabled = 1;
     int rc = ioctlsocket(socketfd, FIONBIO, &enabled);
+#define FLAG_NAME "FIONBIO"
+#elif defined O_NONBLOCK
+    int rc = fcntl(socketfd, F_SETFL, O_NONBLOCK);
+#define FLAG_NAME "O_NONBLOCK"
 #else
     int rc = fcntl(socketfd, F_SETFL, O_NDELAY);
+#define FLAG_NAME "O_NDELAY"
 #endif
     if (rc < 0) {
 	int saved_errno = socket_errno(); // note down in case close hits an error
 	close_fd_or_socket(socketfd);
-	throw Xapian::NetworkError("Couldn't set O_NDELAY", saved_errno);
+	throw Xapian::NetworkError("Couldn't set "FLAG_NAME, saved_errno);
+#undef FLAG_NAME
     }
 
     if (tcp_nodelay) {
