@@ -3,7 +3,7 @@
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001 Hein Ragas
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2014 Olly Betts
  * Copyright 2006 Richard Boulton
  * Copyright 2007 Lemur Consulting Ltd
  *
@@ -1502,6 +1502,12 @@ DEFINE_TESTCASE(crashrecovery1, brass || chert || flint) {
 	TEST_EQUAL(dbr.get_doccount(), 0);
 
 	// Xapian::Database has full set of baseA, no baseB
+	TEST(file_exists(path + "/postlist.baseA"));
+	TEST(file_exists(path + "/record.baseA"));
+	TEST(file_exists(path + "/termlist.baseA"));
+	TEST(!file_exists(path + "/postlist.baseB"));
+	TEST(!file_exists(path + "/record.baseB"));
+	TEST(!file_exists(path + "/termlist.baseB"));
 
 	db.add_document(doc);
 	db.commit();
@@ -1509,6 +1515,12 @@ DEFINE_TESTCASE(crashrecovery1, brass || chert || flint) {
 	TEST_EQUAL(dbr.get_doccount(), 1);
 
 	// Xapian::Database has full set of baseB, old baseA
+	TEST(file_exists(path + "/postlist.baseA"));
+	TEST(file_exists(path + "/record.baseA"));
+	TEST(file_exists(path + "/termlist.baseA"));
+	TEST(file_exists(path + "/postlist.baseB"));
+	TEST(file_exists(path + "/record.baseB"));
+	TEST(file_exists(path + "/termlist.baseB"));
 
 	db.add_document(doc);
 	db.commit();
@@ -1516,6 +1528,12 @@ DEFINE_TESTCASE(crashrecovery1, brass || chert || flint) {
 	TEST_EQUAL(dbr.get_doccount(), 2);
 
 	// Xapian::Database has full set of baseA, old baseB
+	TEST(file_exists(path + "/postlist.baseA"));
+	TEST(file_exists(path + "/record.baseA"));
+	TEST(file_exists(path + "/termlist.baseA"));
+	TEST(file_exists(path + "/postlist.baseB"));
+	TEST(file_exists(path + "/record.baseB"));
+	TEST(file_exists(path + "/termlist.baseB"));
 
 	// Simulate a transaction starting, some of the baseB getting removed,
 	// but then the transaction fails.
@@ -1528,6 +1546,12 @@ DEFINE_TESTCASE(crashrecovery1, brass || chert || flint) {
 
     Xapian::WritableDatabase db(path, Xapian::DB_OPEN);
     // Xapian::Database has full set of baseA, some old baseB
+    TEST(file_exists(path + "/postlist.baseA"));
+    TEST(file_exists(path + "/record.baseA"));
+    TEST(file_exists(path + "/termlist.baseA"));
+    TEST(file_exists(path + "/postlist.baseB"));
+    TEST(!file_exists(path + "/record.baseB"));
+    TEST(!file_exists(path + "/termlist.baseB"));
     Xapian::Database dbr = Xapian::Database(path);
 
     db.add_document(doc);
@@ -1536,6 +1560,12 @@ DEFINE_TESTCASE(crashrecovery1, brass || chert || flint) {
     TEST_EQUAL(dbr.get_doccount(), 3);
 
     // Xapian::Database has full set of baseB, old baseA
+    TEST(file_exists(path + "/postlist.baseA"));
+    TEST(file_exists(path + "/record.baseA"));
+    TEST(file_exists(path + "/termlist.baseA"));
+    TEST(file_exists(path + "/postlist.baseB"));
+    TEST(file_exists(path + "/record.baseB"));
+    TEST(file_exists(path + "/termlist.baseB"));
 
     db.add_document(doc);
     db.commit();
